;------------------------------------------------------------------------------
; Z9001 USB-Modul
; (c) V. Pohlers 2019
; letzte nderung
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

	cpu	z80undoc

p_crt	equ	1	; fast CLS + zustzl. Tasten
p_load	equ	1	; 1x LOAD-Versuch + Patch HELP

	section	modul
	public	load, load1, load4, prepfcb	; fr menukdo.asm

; Die Software nutzt die Eigenschaft des OS, eigene CCP zu schreiben
; (ber '#       '). Es wird eine eigene Suchroutine CPROM und eine eigene
; Programmstartroutine GOCPM genutzt, die die Bnke mit durchsuchen.


;------------------------------------------------------------------------------
; System
;------------------------------------------------------------------------------

CBOS:		EQU	05H		;zentraler BOS-Ruf
;
;CBOS-Rufe
;
;CONSI:		EQU	1		;CONST-Eingabe
;CONSO:		EQU	2		;CONST-Ausgabe
;LISTO:		EQU	5		;LIST-Ausgabe
;CSTS:		EQU	11		;CONST-Status

CSTS:		equ	0F006h		;STATUS CONST
CONSI:		equ	0F009h		;EINGABE ZEICHEN VON CONST
CONSO:		equ	0F00Ch		;AUSGABE ZEICHEN ZU CONST
LISTO:  	equ	0F00Fh

;
IOBYT:		equ	0004h

CONBU:		EQU	0080H		;CCP ZEICHENKETTENPUFFER
STDMA:		EQU	0080H		;STANDARDPUFFER FUER KASSETTE
INTLN:		equ	0100h		; interner Zeichenkettenpuffer
DMA:		EQU	001BH		;ZEIGER AUF KASSETTENPUFFER
PU:		EQU	002FH		;HILFSZELLE (TIME + Status CONST)
WORKA:		EQU	0033H		;HILFSZELLE (ASGN)
PARBU:		EQU	0040H		;HILFSZELLE (wird nur von ALDEV genutzt)

FCB: 		EQU	005Ch 		;Dateikontrollblock
START:		EQU	0071H		;STARTADRESSE

; BDOS
OCRLF:		EQU	0F2FEH
OUTA:		EQU	0F305H
OSPAC:		EQU	0F310H
;
DISPE   	EQU	0F0C0h
GVAL    	EQU	0F1EAh
CPROM		EQU	0F28Eh
LOCK    	EQU	0F2B8h
GETMS   	EQU	0F35Ch
;LOAD1   	EQU	0F526h
REA1		EQU	0F5A6h
COEXT   	EQU	0F5B9h
ERINP		EQU	0F5E2h
ERPAR   	EQU	0F5E6h
ERDIS   	EQU	0F5EAh
WBOOT   	EQU	0F003h	;0F6AEh
;;CONS1		equ	0f758h
;

;25.06.2013
DECO0		equ	0FD33h		; DECODIEREN DER TASTATURMATRIX

;02.03.2019 10:47:50
AUP2		equ	0EFDFh		; Eigentlich Adresse UP2-Treiber fr PUNCH
AUR2		equ	0EFD7h		; Eigentlich Adresse UR2-Treiber fr READER
					; hier f. Re-Init ON_COLD genutzt

;------------------------------------------------------------------------------
; Systemsoftware
;------------------------------------------------------------------------------

		ORG	0C000H
;
PBEG:
		JP	CCP
		DB	"#       ", 0

		JP	CLS
		DB	"CLS     ", 0

		JP	CURSOR
		DB	"C       ", 0

		JP	VER
		DB	"VER     ", 0

		Db	0

;------------------------------------------------------------------------------
; CCP
;------------------------------------------------------------------------------

; Eintritt bei '#        '
CCP:		LD	SP,200H

		if p_crt==1
		ld	hl,CRT
		ld	(0EFCBh),hl
		xor	a
		ld	(keybu1),a
		ld	(keybu2),a
		endif

		LD	DE,aEOS		;XOS-Prompt ausgeben
		LD	C,9
		CALL	5

		;init usb
		call dos

		; on_cold-Treiber via AUR1 starten (USBX/DOSX)
		ld	hl,(AUP2)
		PUSH	HL
           	LD	HL,cold1
            	EX	(SP),HL		;RUECKKEHRADR. KELLERN
          	JP	(HL)
cold1:

		; on_cold-Treiber via AUR2 starten (USBX/DOSX)
		ld	hl,(aur2)
		PUSH	HL
           	LD	HL,cold2
            	EX	(SP),HL		;RUECKKEHRADR. KELLERN
          	JP	(HL)
cold2:

;------------------------------------------------------------------------------
; Eintritt bei RET

GOCPM:		LD	HL,GOCPM
		PUSH	HL			;RUECKKEHRADR. KELLERN
GOCPM1:		LD	HL,STDMA
		LD	(DMA),HL		;STANDARDKASSETTENPUFFER
		LD	A,'>'
		CALL	OUTA		;AUSGABE PROMPT

		call	deinit		;VDIP: ggf. restl. vorhandene Daten abholen
;
		CALL	GETMS			;EINGABE KOMMANDOZEILE
		JP	C, DISPE		;STOP-TASTE ODER FEHLER
		CALL	COEXT			;VORVERARB. EINGABEZEILE
		RET	C			;ZEICHENKETTE LEER
		LD	HL,ERDIS
		PUSH	HL			;ADR. FEHLERROUTINE KELLERN
		CALL	GVAL			;1.PARAMETER HOLEN
		JP	Z, ERPAR		;KEIN NAME
;NAMEN IM SPEICHER SUCHEN
INDV:		PUSH	BC			;TRENNZEICHEN MERKEN
		CALL	CPROM			;NAMEN IM SPEICHER SUCHEN
		POP	BC
		JR	Z, JMPHL		;NAMEN GEFUNDEN (HL)=STARTADR.
;Laden von Kassette
INFIL:		ld	hl,0
		ld	(data),hl
		ld	ix,dfltEXT
		CALL	LOAD1			;DATEI LADEN
		RET	C			;FEHLER BEIM LADEN
		LD	HL,(START)		;(HL)=GELESENE STARTADRESSE
JMPHL:		JP	(HL)			;SPRUNG ZUR AUSFUEHRUNG

aEos:		db 0Bh,14h,1,"EOS",14h,2,0Ah
		db 0Dh,0


;------------------------------------------------------------------------------
;* 	C L O A D   -   KOMMANDO                                   *
; muss leider aus OS kopiert werden, da im OS keine Nutzung der SBOS-Systemrufe :(
;------------------------------------------------------------------------------
;

dfltEXT:	db	"COM"

LOAD:	CALL	GVAL		;NAECHSTEN PARAMETER HOLEN
	RET	Z		;KEIN GUELTIGER NAME
;

;DATEI LADEN OHNE START
LOAD1:	call	prepfcb
	ret	c
LOAD4:
	;CALL	OPENR
	ld	c,13		;OPENR
	call	5
	JR	NC, LOAD5	;KEIN FEHLER
	OR	A
	SCF
	RET	Z		;STOP GEGEBEN
	if p_load==1		;keine Ladewiederholung
	ld	a,0FFh		;<STOP> simulieren
	scf			;immer Ausstieg, keine Wiederholung
	ret
	else		;orig
	CALL	REA1		;AUSG. FEHLERMELD. WARTEN REAKT.
	RET	C		;STOP GEGEBEN
	JR	LOAD4		;WIEDERHOLUNG
	endif
LOAD5:	ld	hl,(data)	;neue aadr?
	ld	a,h
	or	l
	jr	nz,LOAD51
	LD	HL,(FCB+17)	;DATEIANFANGSADRESSE
LOAD51:	LD	(DMA),HL	;NACH ADR. KASSETTENPUFFER
LOA55:	;CALL	READ		;LESEN BLOCK
	ld	c,20		; READ
	call	5
	JR	NC, LOAD6	;KEIN FEHLER
	CALL	REA1		;AUSG. FEHLERMELD. WARTEN REAKT.
	RET	C		;STOP GEGEBEN
	XOR	A
LOAD6:	OR	A
	JR	Z, LOA55	;WEITER BIS DATEIENDE LESEN
	JP	OCRLF
;
MOV:	LD	HL,INTLN+1	;ZWISCHENPUFFER
MOV1:	LD	B,A
MOV2:	LD	A,(HL)
	LD	(DE),A
	INC	HL
	INC	DE
	DJNZ	MOV2
	RET

prepfcb:
	LD	HL,ERPAR
	PUSH	HL
	LD	A,(INTLN)	;PARAMETERLAENGE
	CP	A, 9
	RET	NC		;NAME ZU LANG
	LD	DE,FCB
	LD	A,8
	CALL	MOV		;NAME IN FCB EINTRAGEN
	EX	AF, AF'
	JR	NC, LOAD3	;DATEITYP FOLGT
	EX	AF, AF'
;	LD	HL,4F43H	;STANDARDEINTRAGUNG
;	LD	(FCB+8),HL	;
;	LD	A,'M'		;COM VORNEHMEN
;	LD	(FCB+10),A	;
;	JR	LOA33
	;STANDARDEINTRAGUNG
	push	ix
	pop	hl
	ld	a,3
	JR	LOA31
LOAD3:	LD	A,C
	CP	A, '.'
	POP	HL
	JP	NZ, ERINP	;FALSCHES TRENNZEICHEN
	PUSH	HL
	CALL	GVAL		;PARAMETER HOLEN
	RET	Z		;KEIN GUELTIGER TYP
	LD	A,3
	CP	A, B		;TYP IST ZU LANG
	RET	C
	LD	DE,FCB+8	;TYP IN FCB EINTRAGEN
;	CALL	MOV
	LD	HL,INTLN+1	;ZWISCHENPUFFER
loa31:	CALL	MOV1
LOA33:	POP	HL
	EX	AF, AF'		;'
	JP	NC, ERINP	;ZU VIELE PARAMETER
	ccf
	ret

;-------------------------------------------------------------------------------
;CLS
;-------------------------------------------------------------------------------
;
CLS:		ld	a,0ch
		call	OUTA
		or	a
		ret
;
;CURSOR und VER ans Ende verschoben, damit Code noch vor 100h-Grenze endet
;spart Bytes ...

;-------------------------------------------------------------------------------
;CURSOR on/off
;-------------------------------------------------------------------------------
;
CURSOR:		ld	hl,0efc8h
		ld 	a,(hl)
		XOR	a, 00100000B		; "Farb-RAM existiert" togglen (E800-EBFF)
		ld 	(hl),a
		ret

;-------------------------------------------------------------------------------

VER:		ld	de, vertxt
		ld	c,9
		call	5
		ret

vertxt:		db	"USB-OS und Tools"
		db	13,10
		db	"(c) V.Pohlers, Neustadt i.H., "
		db	DATE
		db	13,10,0

		endsection

; noch 200 Byte frei fr Erweiterung ...

;-------------------------------------------------------------------------------
;Einbindung CRT-TReiber-Erweiterung
;-------------------------------------------------------------------------------

		if p_crt==1
		section crtdrv
		public	   crt,keybu1,keybu2
crtpart	eval	"CO"		
		include system\crtdrv.asm
;CRT	equ	0F8F1h
		endsection
		endif

;-------------------------------------------------------------------------------
;Einbindung USB
;DOS, CAOS, USB, DIR, CD
;-------------------------------------------------------------------------------

		align	100h

dos:		section	usb
		public	deinit
		include	system\usbos.asm
		endsection

;-------------------------------------------------------------------------------
;Einbindung Monitorzusatzkommandos
;MENU, DUMP, FILL, TRANS, RUN, IN, OUT, MEM, EOR, LOAD, SAVE, FCB
;-------------------------------------------------------------------------------

		align	100h

		section eoskdo
		public	cload, csave, data, KDOPAR, OUTHX

		include	system\menukdo.asm

; Kommandos vom Z1013

; Arbeitsspeicher
currbank	equ	0042h		; aktuelle Bank
firstent:	equ	currbank+1	; temp. Zelle f. Menu
DATA:		equ	firstent+1	; Konvertierungsbuffer
ARG1:		equ	DATA+2		; 1. Argument
ARG2:		equ	ARG1+2		; 2. Argument
ARG3:		equ	ARG2+2		; 3. Argument
ARG4:		equ	ARG3+2		; 4. Argument

		include	system\syskdo.asm

		endsection

;-------------------------------------------------------------------------------
;weiteres
;HELP, CALL5 Ext.
;-------------------------------------------------------------------------------

		;;align	100h

		section hlpkdo
		public help
		include	system\helpkdo.asm
		endsection

		; CALL5 Zustzliche Funktionen
		include	system\usbos2.asm

;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;Einbindung CRT-TReiber-Erweiterung
;-------------------------------------------------------------------------------

		if p_crt==1
		section crtdrv2
		public	ci0
crtpart	eval	"CI"
		include system\crtdrv.asm
		endsection
		endif

;-------------------------------------------------------------------------------
; SYSINFO dazupacken

	if 1=1
	if $ > 0D000h
		error "Modul berschneidet sich mit SYSINFO!"
	endif

		org	0D000h

		section sysinfo

		binclude	system\sysinfo_loader.bin
		endsection
	endif		
;-------------------------------------------------------------------------------
		END
